const int LED_PIN = 2;
const int BUTTON_PIN = 7;

class Button{
  public:
  unsigned long t_of_state_2;
  unsigned long t_of_button_change;    
  unsigned long debounce_time;
  unsigned long long_press_time;
  int pin;
  int flag;
  bool button_pressed;
  int state; // Public for the sake of convenience
  Button(int p) {
  flag = 0;  
    state = 0;
    pin = p;
    t_of_state_2 = millis(); //init
    t_of_button_change = millis(); //init
    debounce_time = 1;
    long_press_time = 100;
    button_pressed = 0;
  }
  void read() {
    int button_state = digitalRead(pin);  
    button_pressed = !button_state;
  }
  int update() {
    read();
    flag = 0;
    if (state==0) { // Unpressed, rest state
      if (button_pressed) {
        state = 1;
        t_of_button_change = millis();
      }
    } else if (state==1) { //Tentative pressed
      if (!button_pressed) {
        state = 0;
        t_of_button_change = millis();
      } else if (millis()-t_of_button_change >= debounce_time) {
        state = 2;
        t_of_state_2 = millis();
      }
    } else if (state==2) { // Short press
      if (!button_pressed) {
        state = 4;
        t_of_button_change = millis();
      } else if (millis()-t_of_state_2 >= long_press_time) {
        state = 3;
      }
    } else if (state==3) { //Long press
      if (!button_pressed) {
        state = 4;
        t_of_button_change = millis();
      }
    } else if (state==4) { //Tentative unpressed
      if (button_pressed && millis()-t_of_state_2 < long_press_time) {
        state = 2; // Unpress was temporary, return to short press
        t_of_button_change = millis();
      } else if (button_pressed && millis()-t_of_state_2 >= long_press_time) {
        state = 3; // Unpress was temporary, return to long press
        t_of_button_change = millis();
      } else if (millis()-t_of_button_change >= debounce_time) { // A full button push is complete
        state = 0;
        if (millis()-t_of_state_2 < long_press_time) { // It is a short press
          flag = 1;
        } else {  // It is a long press
          flag = 2;
        }
      }
    }
    return flag;
  }
};


long last_time;

// the setup function runs once when you press reset or power the board
void setup() {
  // initialize digital pin LED_BUILTIN as an output.
  pinMode(LED_PIN, OUTPUT);
  pinMode(BUTTON_PIN, INPUT_PULLUP);

  last_time = millis();
}

Button button = (BUTTON_PIN);
int led_state = 0; // 0 = off, 1 = on

// the loop function runs over and over again forever
void loop() {
  int buttonFlag = button.update();
  if (buttonFlag == 1){
    led_state = 1 - led_state;
  }

  if (led_state == 0){
    digitalWrite(LED_PIN, LOW);
  } else {
    digitalWrite(LED_PIN, HIGH);
  }
}
